import { CSVMaker } from '.'

export class DefaultCSVMaker implements CSVMaker {
  private findKeysFlatten(item: any): Array<string> {
    const keys: Array<string> = []
    for (const key of Object.keys(item)) {
      if (item[key] instanceof Object) {
        for (const subkey of this.findKeysFlatten(item[key])) {
          keys.push(key + '.' + subkey)
        }
      } else {
        keys.push(key)
      }
    }
    return keys
  }

  private getAttribute(item: any, key: string): any {
    const segments = key.split('.')
    let currentItem = item
    for (const segment of segments) {
      if (!currentItem) return '-1'
      currentItem = currentItem[segment]
    }
    return currentItem
  }

  toCSV(data: Array<any>, columnDelimiter = ',', lineDelimiter = '\n'): string {
    // Finding object with most attributes (not recursively)
    let itemWithMostAttributes = new Object()
    for (const item of data) {
      if (Object.keys(itemWithMostAttributes).length < Object.keys(item).length) {
        itemWithMostAttributes = item
      }
    }

    // Extracting all keys of this object (recursively)
    const keys: Array<string> = this.findKeysFlatten(itemWithMostAttributes)

    // Preparing CSV header with keys
    let result = keys.join(columnDelimiter) + lineDelimiter

    // For each object, we push one value per key by reading the object recursively
    for (const item of data) {
      const linedata: Array<string> = []
      for (const key of keys) {
        const value = this.getAttribute(item, key)
        linedata.push(value === undefined || value === null ? '-1' : value.toString())
      }
      // Preparing CSV line
      result += linedata.join(columnDelimiter) + lineDelimiter
    }
    return result
  }

  toCSVWithDefaultDelimiters(data: any[]): string {
    return this.toCSV(data, ',', '\n')
  }
}
