import axios, { AxiosInstance } from 'axios'
import { Problem } from '../exceptions'
import { HTTPClient, HTTPClientConfig, HTTPResponse, RequestConfig } from './httpClient.interface'

export class AxiosHTTPClient implements HTTPClient {
  private instance: AxiosInstance

  constructor(readonly config: Partial<HTTPClientConfig> = {}) {
    this.instance = axios.create({
      baseURL: config.host + '/api/v4',
      headers: {
        Authorization: `Bearer ${config.accessToken}`,
      },
    })
  }

  private checkConfiguration() {
    if (!this.config.host) {
      throw new Problem('No configuration found. Please configure the Gitlab host to use.')
    }
  }

  async get<T>(path: string, config?: RequestConfig): Promise<HTTPResponse<T>> {
    this.checkConfiguration()
    const { data, headers, status } = await this.instance.get(path, config || {})
    return { data, headers, status }
  }

  async post<T>(path: string, config?: RequestConfig): Promise<HTTPResponse<T>> {
    this.checkConfiguration()
    const { data, headers, status } = await this.instance.post(path, config?.body, config || {})
    return { data, headers, status }
  }
}
