import { DataObject, StorageProvider } from '../../core/storage'
import { OnProvideProjectData } from './projectData.interface'

export class ProjectDataCollectWriter {
  constructor(private storageProvider: StorageProvider, private providers: OnProvideProjectData[] = []) {}

  addProvider(provider: OnProvideProjectData) {
    this.providers.push(provider)
  }

  async write({ projectId, path }: { projectId: number; path: string }): Promise<void> {
    const storage = this.storageProvider.forPath(path)

    // Collect project infos
    const infos: DataObject[] = []
    for (const provider of this.providers) {
      infos.push(...(await provider.onProvideProjectData(projectId)))
    }

    // Store
    storage.set(...infos)
  }
}
